@@ -8,12 +8,10 @@ for the of DeepSR object (DeepSR.py)
 Developed  by
         Hakan Temiz             htemiz@artvin.edu.tr
 
-Version : 0.0.51
-History :
-
 """
 
 import argparse
+import sys
 
 
 parser = argparse.ArgumentParser(description="A Python framework for Image Super Resolution.")
@@ -215,6 +213,17 @@ parser.add_argument("--scale", type=int, default=2,
 parser.add_argument('--seed', type=int, default=19, help='Integer. Seed number of number generators to be used for \
                     re-producable deterministic models.')
 
+parser.add_argument("--realcase", action="store_true", default=False,
+                    help="Boolean. Default True. DeepSR class also stores its value (True, or False) as a member with the same name."
+                         "True, to generate simulated Low resolution images from input images which"
+                         "are considered as high resolution. False, to separately feed low and high resolution versions of images."
+                         "In this case, the path to low resolution images should be given by the command argument --traindir_lr."
+                         "LR to HR counter parts are picked from their own paths with the same order, and fed to the DL model in order"
+                         "The path to low resolution images is saved in the class member of DeepSR with the name 'traindir_lr'."
+                         "When simulate variable is False, pre-process step for generating simulated low resolution images is "
+                         "skipped."
+                        )
+
 parser.add_argument("--shuffle",  action="store_true", default=True,
                     help="Boolean. Input images or images patches to be shuffled  if it is given \
                     (or set to True in DeepSR object), in order to ensure randomness. Default is True.")
@@ -254,6 +263,9 @@ parser.add_argument("--train", action="store_true", default=False,
 parser.add_argument("--traindir", type=str, default='',
                     help="String. The directory path where the training images in.")
 
+parser.add_argument("--traindir_lr", type=str, default='',
+                    help="String. The directory path where the training images in.")
+
 parser.add_argument("--train_with_generator",  action="store_true", default=False,
                     help="Command to run training with generator for huge data sizes, after class is built. \
                     to be implemented in future versions.")
@@ -278,7 +290,7 @@ parser.add_argument("--workingdir", type=str, default='',
 
 def getArgs():
 
-    return vars(parser.parse_args())
+    return vars(parser.parse_known_args()[0]) # parse_known_args returns args, unknown_args
 
 
 """   
