@@ -7,9 +7,6 @@ and for image processing.
 Developed  by
         Hakan Temiz             htemiz@artvin.edu.tr
 
-    version : 0.0.51
-    history :
-
 """
 
 import h5py
@@ -33,8 +30,8 @@ from tqdm import tqdm
 
 # measures
 from sporco.metric import snr, bsnr, pamse, gmsd, isnr
-from skimage.measure import compare_psnr, compare_ssim as SSIM
-from skimage.measure import compare_mse, compare_nrmse
+from skimage.metrics import peak_signal_noise_ratio, structural_similarity as SSIM
+from skimage.metrics import mean_squared_error, normalized_root_mse
 import sewar
 from skvideo.measure import  mad, niqe #, brisque , msssim, strred
 
@@ -288,7 +285,7 @@ def measure_images(image_path, scale=2, metrics= "ALL", channels= 1, decimation=
                    methods= METHODS, border=0, pad=0, save_images=False, mode=None):
     """
         Used for Image Quality Assessment (IQA) of images in given folder path. Ground truth images
-        maintained from the images in the given path. The images to be evaluated generated using
+        maintained from the images from the path. The images to be evaluated generated using
         given interpolation methods, and then, IQA is maintained between ground truth images and
         result images.
 
@@ -369,7 +366,7 @@ def measure_images(image_path, scale=2, metrics= "ALL", channels= 1, decimation=
 
 
 def calc_multi_metrics_of_image(img_ground, image, border=0, channels=1,
-        metrics=["PSNR", "SSIM"]):
+                                metrics=["PSNR", "SSIM"], fn_user_metrics=None):
     """
     Calculates any of following image quality indexes of an image from itself or from a reference ground truth image:
 
@@ -381,6 +378,22 @@ def calc_multi_metrics_of_image(img_ground, image, border=0, channels=1,
 
     Images to be converted to uint8 space. Intensity values should not be normalized
     So, values must be between 0 and 255 inclusive
+    Addition to given metrics by the parameter 'metrics', user-defined metrics can
+    also be computed. For this, user should provide the function handle with the parameter
+    'fn_user_metrics'. This function should take two numpy's ndarray and return
+    a dictionary with (name, value) pairs for each computed metric. The should look like this:
+        :example:
+        def fn_user_metrics(ground_img, output_img):
+            res_dict = dict()
+            for each metric in user-defined_metrics:
+                res_dict[<name of metric>] = computed value for the metric
+
+            return res_dict
+
+    This function can be defined by the user with the same name (fn_user_metrics) in the model file.
+    In this case, this function is added as a member method to the DeepSR class while reading
+    the model file. It is not necessary to provide the function handle as a parameter in this case.
+    The existence of this member method is checked, and if it is found,it is run automatically in any case.
 
     :return: a dictionary with image quality measures
 
@@ -391,6 +404,9 @@ def calc_multi_metrics_of_image(img_ground, image, border=0, channels=1,
     :param border: cropping size from borders of image
     :param channels: number of color channels of image to be processed
     :param img_ground: reference image
+    :param fn_user_metrics: A function computing the discrepancy between reference image
+         and output image by user-defined metric(s). This function returns a dictionary
+         with name,value pairs for each metric computed inside the function.
     :param metrics: a list of combinations of following image quality indexes;
 
 
@@ -542,7 +558,13 @@ def calc_multi_metrics_of_image(img_ground, image, border=0, channels=1,
         """
 
         if "GMSD" in metrics or "ALL" in metrics:
-            results["GMSD"] =  gmsd(img_ground, image, rescale=True, returnMap=False)
+            if len(image.shape) > 2:
+                print('An 2d array is required for GMSD but 3d given. The mean of GMSD values over all channels taken.')
+                gmsds = [gmsd(img_ground[:,:,i], image[:,:,i], rescale=True, returnMap=False) for i in range(image.shape[2])]
+
+                results['GMSD'] = np.mean(gmsds)
+            else:
+                results["GMSD"] =  gmsd(img_ground, image, rescale=True, returnMap=False)
         #
         # if "HAARPSI" in metrics or "ALL" in metrics:
         #     results["HAARPSI"] = haar_psi(img_ground, image, False)[0]
@@ -556,10 +578,15 @@ def calc_multi_metrics_of_image(img_ground, image, border=0, channels=1,
             #results["ISNR"] =  isnr()
         """
         if "MAD" in metrics or "ALL" in metrics:
-            results["MAD"] = mad.mad(img_ground, image)[0]
+            if len(image.shape) > 2:
+                print('Given image contains 3 channels. MAD expects only one (luminance).'
+                      'The first channel is being used for computation of MAD')
+                results["MAD"] = mad.mad(img_ground[:,:,0], image[:,:,0])[0]
+            else:
+                results["MAD"] = mad.mad(img_ground, image)[0]
 
         if "MSE" in metrics or "ALL" in metrics:
-            results["MSE"] = compare_mse(img_ground, image)
+            results["MSE"] = mean_squared_error(img_ground, image)
 
         if "MSSSIM" in metrics or "ALL" in metrics:
             results["MSSSIM"] = sewar.msssim(img_ground, image)
@@ -572,9 +599,14 @@ def calc_multi_metrics_of_image(img_ground, image, border=0, channels=1,
             if (image.shape[0] <= 192 or image.shape[1] <= 192):
                 print("\nOne of dimensions of Image is less than 192! NIQE requires",
                   "192 pixel at minimum in any dimension. The value of NIQE ",
-                  "is being set to 'None'")
+                  "is set to 'None'")
+
                 results["NIQE"]= None
 
+            elif len(image.shape) > 2:
+                    print('Given image contains 3 channels. NIQE expects only one (luminance).'
+                          'The first channel is being used for computation of NIQE')
+                    results["NIQE"] = niqe(image[:,:,0])[0]
             else:
                 results["NIQE"] = niqe(image)[0]
         #
@@ -582,7 +614,7 @@ def calc_multi_metrics_of_image(img_ground, image, border=0, channels=1,
         #     results["NQM"] = nqm(img_ground, image)
 
         if "NRMSE" in metrics or "ALL" in metrics:
-            results["NRMSE"] = compare_nrmse(img_ground, image)
+            results["NRMSE"] = normalized_root_mse(img_ground, image)
 
         if "PAMSE" in metrics or "ALL" in metrics:
             results["PAMSE"] = pamse(img_ground, image, rescale=True)
@@ -591,7 +623,7 @@ def calc_multi_metrics_of_image(img_ground, image, border=0, channels=1,
         #     results["PBVIF"] = pbvif(img_ground, image)
 
         if "PSNR" in metrics or "ALL" in metrics:
-            results["PSNR"] = compare_psnr(img_ground, image)
+            results["PSNR"] = peak_signal_noise_ratio(img_ground, image)
 
         if "RASE" in metrics or "ALL" in metrics:
             results["RASE"] = sewar.rase(img_ground, image)
@@ -623,6 +655,27 @@ def calc_multi_metrics_of_image(img_ground, image, border=0, channels=1,
             results["WSNR"] = wsnr(img_ground, image)
         """
 
+        # ---------------------------------------------------------------
+        # -------------------  USER DEFINED METRICS  --------------------
+        # User-defined metrics can also be calculated. These metrics can be
+        # given either with fn_user_metrics parameter, or
+        # by providing the function named fn_user_metrics in the model file.
+        # The function is expected to return a dictionary containing name, value
+        # pairs of each calculated metric.
+
+        # If the user-defined functions is given with the parameter of this method
+        if fn_user_metrics is not None:
+
+            if not callable(fn_user_metrics):  # check if a valid function is given
+                raise Exception("The parameter 'fn_user_metrics' is not a callable object."
+                                "This should be a function returning the <name, computed value> pairs for "
+                                "each user-defined metrics")
+
+            res_dict = fn_user_metrics(img_ground, image)
+            for name, value in res_dict.items(): #iterate each calculated metric
+                results[name] = value
+        # ---------------------------------------------------------------
+
         return results
     else:
         print('\nGround Image is of shape ', img_ground.shape, ',  but the image is of shape ', image.shape)
@@ -666,7 +719,7 @@ def calc_metrics_of_image(img_ground, image, border=0, channels=1, metrics=["PSN
         if image.dtype != 'uint8':
             image = image.astype(np.uint8)
 
-        psnr = compare_psnr(img_ground, image)
+        psnr = peak_signal_noise_ratio(img_ground, image)
         ssim = SSIM(img_ground, image, multichannel=channels)
         return psnr, ssim
     else:
@@ -1421,16 +1474,19 @@ def remove_if_exist(file_name):
         remove(file_name)
 
 
-def augment_image(image, options):
+def augment_image(image, options, fn_user_augmentation=None):
     """
     Augments the input image. Augmentation process covers rotation of 90, 180, 270 degrees
     and transposition of upside down, left to right and mix of the both. Transposition
     styles are defined by options parameter. options can be any or all of
     [90,180,270, 'flipud', 'fliplr', 'flipudlr'].
     Takes scikit-image as input (numpy ndarray) and returns augmented images.
+    User defined augmentation procedure given via the parameter, fn_augment, is also performed.
 
     :param image:
     :param options: a list consists of [90,180,270, 'flipud', 'fliplr', 'flipudlr']
+    :param fn_user_augmentation: User-defined function for image augmentation. This function
+        takes an image as ndarray, and returns a list of images.
     :return: returns transposed and rotated versions of original image,
      as well as the original image, totally 7 images.
     """
@@ -1459,47 +1515,22 @@ def augment_image(image, options):
     if 'flipudlr' in options: # flip upside down and then left to right
         images.append(np.fliplr(np.flipud(image)))
 
-    """
-    for item in options:
-        if 'scale' in item:
-            scales = item.split(' ')[1:]
+    # user-defined augmentation function
+    if fn_user_augmentation is not None :
+        if not callable(fn_user_augmentation): # check if a valid function is given
+            raise Exception("The parameter 'fn_user_augmentation' is not callable!"
+                            "This should be a function returning a list of images (as ndarrays) "
+                            "generated in an user-defined augmentation process.")
+        else:
+            image_list = fn_user_augmentation(image)
 
-        elif 'noise' in item:
-            noises = item.split(' ')[1:]
+            if isinstance(image_list, list):
+                images.extend(image_list)
+            else:
+                images.append(image_list)
 
-        elif 'interp' in item:
-            interps = item.split(' ')[1:]
-
-    n_scales = len(scales)
-    n_noises = len(noises)
-    n_interps= len(interps)
-
-    if n_scales <1: n_scales=1
-    if n_noises <1: n_noises=1
-    if n_interps<1: n_interps=1
-
-    n_total_images = n_scales * n_noises * n_interps + len(images)
-
-    all_images = [None] * n_total_images
-
-    i = 0
-    for im in images:
-        all_images[i] = im
-
-        if len(interps) > 0:
-            f
-
-    for i in range(n_total_images):
-        for im in images:
-            all_images[i] = im
-
-            for interp in interps:
-                for scale in scales:
-                    for noise in noises:
-
-                        pass
-
-    """
+    # remove None elements
+    images = [i for i in images if i is not None]
     return images
 
 
